//-----------------------------------------------------------------
//GL2DVerTex Class (Task Class 継承)
//ポリゴンの生成、更新、描画を簡単にさせるためのクラス
//Copyright：TOMY
//Ver:1.000
//-----------------------------------------------------------------
package jp.denpa.tomy.opengl_es.wrap;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.math.*;

import javax.microedition.khronos.opengles.GL10;



public class GL2DVertex {

	public Vector2 center;			//中心座標
	public Vector2 size;			//ポリゴンサイズ
	public float   rad;				//回転角度（radian）

	private float a,r,g,b;			//カラー+アルファ

	//UVの設定用
	private float[] uv = new float[8];
	private boolean	turnFlg;		// テクスチャのUVを左右反転するか

	//壊れることのない情報(配列定数)
	private final static float[] dx={-1.0f, 1.0f, 1.0f, -1.0f};
	private final static float[] dy={-1.0f, -1.0f, 1.0f, 1.0f};

	//計算が分かりやすい様にする為の変数
	private float[] tx = new float[4];
	private float[] ty = new float[4];



	public void Init(){		//初期化



		turnFlg		= false;
		rad			= 0;
		//色の初期設定
		a=1.0f;
		r=1.0f;
		g=1.0f;
		b=1.0f;

		//UVの初期化
		float w = 1.0f;
		float h = 1.0f;

		float[] tu = new float[2];
		float[] tv = new float[2];

		tu[0]	= w;
		tv[0]	= h;
		tu[1]	= 0;
		tv[1]	= 0;

		if(turnFlg==true){			//反転フラグがtrueの時の処理
			float temp=tu[1];
			tu[1]=tu[0];
			tu[0]=temp;
		}

		//UV座標
		uv[0] = tu[0];
		uv[1] = tv[1];		//左上

		uv[2] = tu[1];
		uv[3] = tv[1];		//右上

		uv[4] = tu[1];
		uv[5] = tv[0];		//右下

		uv[6] = tu[0];
		uv[7] = tv[0];		//左下



	}

	public void SetUV(int cx,int cy,int nx,int ny){			//UV座標の変更１(等分割)
		float w = 1.0f/cx;
		float h = 1.0f/cy;

		float[] tu = new float[2];
		float[] tv = new float[2];

		tu[0]	= w * nx;
		tv[0]	= h * (ny+1);
		tu[1]	= w * (nx+1);
		tv[1]	= h * ny;

		if(turnFlg==true){			//反転フラグがtrueの時の処理
			float temp=tu[1];
			tu[1]=tu[0];
			tu[0]=temp;
		}
		uv[0] = tu[0];
		uv[1] = tv[1];		//左下

		uv[2] =	tu[1];
		uv[3] = tv[1];		//左上

		uv[4] = tu[1];
		uv[5] = tv[0];		//右上

		uv[6] = tu[0];
		uv[7] = tv[0];		//右下

	}

//	public void SetUV(Vector2 UvPos1,Vector2 UvPos2,Vector2 UvPos3,Vector2 UvPos4,GLTexture Tex){		//UV座標の変更２ (ピクセル座標指定)（未完成）
//		float[] tu = new float[2];
//		float[] tv = new float[2];
//
//		tu[0]	= (UvPos1.x / GLTexture.texSize.x);
//		tv[0]	= (UvPos1.y / GLTexture.texSize.y);
//		tu[1]	= tu[0] + (UvPos1.x / GLTexture.texSize.x);
//		tv[1]	= tv[0] + (UvPos1.y / GLTexture.texSize.y);
//
//		if(turnFlg==true){			//反転フラグがtrueの時の処理
//			float temp=tu[1];
//			tu[1]=tu[0];
//			tu[0]=temp;
//		}
//		uv[0] = tu[0];
//		uv[1] = tv[1];		//左下
//
//		uv[2] =	tu[1];
//		uv[3] = tv[1];		//左上
//
//		uv[4] = tu[1];
//		uv[5] = tv[0];		//右上
//
//		uv[6] = tu[0];
//		uv[7] = tv[0];		//右下
//
//	}

	public void SetColor(float A,float R,float G,float B){		//色の設定
		a=A;r=R;g=G;b=B;
	}


	public void Draw(int TexName){		//テクスチャありの描画

		float[] verX = new float[4];
		float[] verY = new float[4];

		//計算の簡略化用
		for(int i=0;i<4;i++){
			tx[i] = size.x / 2 * dx[i];
			ty[i] = size.y / 2 * dy[i];
		}
		for(int i=0;i<4;i++){
			verX[i] = (float) (tx[i] * Math.cos(rad) - ty[i] * Math.sin(rad) + center.x);
			verY[i] = (float) (tx[i] * Math.sin(rad) + ty[i] * Math.cos(rad) + center.y);
		}

		//位置情報
		float vertices[] = {			//x,y,zの順に定義
				verX[0],verY[0],			//左上
				verX[1],verY[1],			//右上
				verX[2],verY[2],			//右下
				verX[3],verY[3],			//左下
		};

		float colors[] = {				//頂点色情報
			r,g,b,a,
			r,g,b,a,
			r,g,b,a,
			r,g,b,a
		};

		FloatBuffer Vertices 	= GraphicUtil.makeFloatBuffer(vertices);
		FloatBuffer Colors		= GraphicUtil.makeFloatBuffer(colors);
		FloatBuffer Uv 			= GraphicUtil.makeFloatBuffer(uv);



		// テクスチャ有り
		Global.gl.glEnable(GL10.GL_TEXTURE_2D);						//テクスチャ使用有効化
	   	Global.gl.glEnable(GL10.GL_BLEND);						   	//アルファブレンディングの有効化
		Global.gl.glBindTexture(GL10.GL_TEXTURE_2D, TexName);		//テクスチャセット

		Global.gl.glVertexPointer(2, GL10.GL_FLOAT, 0, Vertices);	//OpenGL ESに頂点バッファを関連付ける(コピーではないことに注意)
		Global.gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);

		Global.gl.glColorPointer(4, GL10.GL_FLOAT, 0, Colors);		//OpenGL ESに色情報バッファを関連付ける
		Global.gl.glEnableClientState(GL10.GL_COLOR_ARRAY);

		Global.gl.glTexCoordPointer(2, GL10.GL_FLOAT, 0, Uv);		//OpenGL ESにテクスチャマッピング用頂点バッファを関連付ける(コピーではないことに注意)
		Global.gl.glEnableClientState(GL10.GL_TEXTURE_COORD_ARRAY);

		Global.gl.glDrawArrays(GL10.GL_TRIANGLE_FAN,0,4);			//描画

		Global.gl.glDisableClientState(GL10.GL_TEXTURE_COORD_ARRAY);
		Global.gl.glDisable(GL10.GL_BLEND);			   				//アルファブレンディングの無効化
		Global.gl.glDisable(GL10.GL_TEXTURE_2D);					//テクスチャ使用無効化


	}

	public void Draw(){


		float[] verX = new float[4];
		float[] verY = new float[4];

		//計算の簡略化用
		for(int i=0;i<4;i++){
			tx[i] = size.x / 2 * dx[i];
			ty[i] = size.y / 2 * dy[i];
		}
		for(int i=0;i<4;i++){
			verX[i] = (float) (tx[i] * Math.cos(rad) - ty[i] * Math.sin(rad) + center.x);
			verY[i] = (float) (tx[i] * Math.sin(rad) + ty[i] * Math.cos(rad) + center.y);
		}

		//位置情報
		float vertices[] = {			//x,y,zの順に定義
				verX[0],verY[0],			//左上
				verX[1],verY[1],			//右上
				verX[2],verY[2],			//右下
				verX[3],verY[3],			//左下
		};

		float colors[] = {				//頂点色情報
			r,g,b,a,
			r,g,b,a,
			r,g,b,a,
			r,g,b,a
		};

//		//OpenGLはビッグエンディアンではなく
//		//CPUごとの”ネイティブエンディアン”で数値を伝える必要がある。
//		//その為Javaヒープを直接的には扱えず、
//		//”Java.nio”配下のクラスへ一度値を格納する必要がある。（面倒くせぇ）
//		ByteBuffer bb=ByteBuffer.allocateDirect(positions.length * 4);  //GL_ESが扱えるのはallocateDirectで確保した領域のみ
//		bb.order(ByteOrder.nativeOrder());		//実行環境に合わせて最適な値を指定できるようにnativeOrderを使う
//		FloatBuffer fb = bb.asFloatBuffer();	//確保したバッファをfloatに格納(新たにメモリを確保しているわけではない)
//		fb.put(positions);						//バッファをfloat配列に転送(この時点でエンディアンに関する変換はもう終わっている)
//		fb.position(0);							//putすると書き込み位置が一つ進められるのでpositionで最初に戻す
//上の処理を簡略化したの↓
		FloatBuffer Vertices 	= GraphicUtil.makeFloatBuffer(vertices);
		FloatBuffer Colors		= GraphicUtil.makeFloatBuffer(colors);

		Global.gl.glVertexPointer(2, GL10.GL_FLOAT, 0, Vertices);	//OpenGL ESに頂点バッファを関連付ける(コピーではないことに注意)
		Global.gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
		Global.gl.glColorPointer(4, GL10.GL_FLOAT, 0, Colors);		//OpenGL ESに色情報バッファを関連付ける
		Global.gl.glEnableClientState(GL10.GL_COLOR_ARRAY);

		Global.gl.glDrawArrays(GL10.GL_TRIANGLE_FAN, 0, 4);		//描画

}


	public void SetTurnFlg(boolean turnflg)	{ turnFlg 	 = turnflg;	}
//	public boolean GetTurnFlg()				{ return turnFlg;		}

}
